#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <termios.h>
#include <time.h>
#include <fcntl.h>

typedef char Screen_Field_T;

#define SCREEN_ROWS 10
#define SCREEN_COLUMNS 20

Screen_Field_T screen[SCREEN_ROWS][SCREEN_COLUMNS];

void setpos_term(int row, int column) {
  // f   HVP       Move cursor to the indicated row, column.
  char cmd[20];
  sprintf(cmd, "\x1b[%02d;%02df", row, column);
  write(1, cmd, 8);  
}

void clear_term() {
  // ESC [ 2 J: erase whole display.
  write(1, "\x1b[2J", 4);
  // d   VPA       Move cursor to the indicated row, current column.
  write(1, "\x1b[1d", 4);
  // G   CHA       Move cursor to indicated column in current row
  write(1, "\x1b[1G", 4);
}

void display_border() {
  useconds_t usecs = 10000;
  
  for (int i = 0; i < SCREEN_COLUMNS+2; i++) {
    write(1, "+", 1);
    usleep(usecs);
  }
  for (int i = 1; i < SCREEN_ROWS+2; i++) {
    setpos_term(i, SCREEN_COLUMNS+2);
    write(1, "+", 1);
    usleep(usecs);
  }
  for (int i = SCREEN_COLUMNS+2; i >=1 ; i--) {
    setpos_term(SCREEN_ROWS+2, i);
    write(1, "+", 1);
    usleep(usecs);
  }
  for (int i = SCREEN_ROWS+2; i>=1; i--) {
    setpos_term(i, 0);
    write(1, "+", 1);
    usleep(usecs);
  }
}

void clear_screen() {
  for (int i = 0; i < SCREEN_ROWS; i++) 
    for (int j = 0; j < SCREEN_COLUMNS; j++)
      screen[i][j] = ' ';
}

void hide_symbols() {
  for (int i = 0; i < SCREEN_ROWS; i++) 
    for (int j = 0; j < SCREEN_COLUMNS; j++)
      if (screen[i][j] != ' ') {
	setpos_term(i+2,j+2);
	write(1, "?", 1);
      }
}

void display_field() {
  for (int i = 0; i < SCREEN_ROWS; i++) 
    for (int j = 0; j < SCREEN_COLUMNS; j++) {
      setpos_term(i+2, j+2);
      write(1, &screen[i][j], 1);
    }

}

int main(int argc, char **argv) {
  static struct termios old_termios, new_termios;
  tcgetattr(STDIN_FILENO, &old_termios);
  cfmakeraw(&new_termios);
  tcsetattr(STDIN_FILENO, TCSANOW, &new_termios);
  int r; int c;
  r = 2; c = 2;

  srand(time(NULL));
  int level;
  if (argc > 1) {
    level = atoi(argv[1]);
  }
  else
    level = 1;
 next_level:
  clear_term();
  display_border();
  clear_screen();


  char symbol = 'A';
  char last_sym;
  for (int i = level; i > 0; i--) {
    while (1) {
      int row = (rand() % SCREEN_ROWS);
      int column = (rand() % SCREEN_COLUMNS);
      if (screen[row][column] == ' ' && row != r-2 && column != c-2) {
	screen[row][column] = symbol++;
	break;
      }
    }
  }
  last_sym = symbol-1;
  symbol = 'A';
  display_field();
  while ( getchar() != ' ');
  hide_symbols();

  // <  44
  // /\ 41
  // >  43
  // \/ 42
  useconds_t usecs = 80000;
  while(1) {
  
    //printf("r = %d e c=%d\n",r,c);
    setpos_term(r,c);
    if (screen[r-2][c-2] != ' ') {
      if (screen[r-2][c-2] == symbol) {
	write(1, &screen[r-2][c-2], 1);
	screen[r-2][c-2] = ' ';
	  if (symbol == last_sym) {
	  level++;
	  goto next_level;
	}
	symbol++;
	/*
	if (('A' - symbol)%3 == 0) {
	    display_field();
	    sleep(2);
	    hide_symbols();
	
	    }
	*/
      } else {
	break;
      }
    } else {
      write(1, "@", 1);
      usleep(usecs);
    }
    char cmd[6];
    int flags = fcntl(0, F_GETFL, 0);
    fcntl(0, F_SETFL, flags | O_NONBLOCK);
    int rb = read(0, &cmd, 6);
    int last;
    setpos_term(r,c);
    write(1, " ", 1);
    if (rb != 3)
      cmd[2] = last;
    else last = cmd[2];
    switch (cmd[2]) {
    case 0x44:
      if (c>2) c--;
      break;
    case 0x41:
      if (r>2) r--;
      break;
    case 0x43:
      if (c <= SCREEN_COLUMNS) c++;
      break;
    case 0x42:
      if (r <= SCREEN_ROWS) r++;
      break;
    default:
      ;
    }
  }
  tcsetattr(STDIN_FILENO, TCSANOW, &old_termios);
  clear_term();
  printf("[+] You lose at level %d\n", level);

  return 0;
}
